@@ -1,10 +1,10 @@
 #!/usr/bin/perl
 
 ########################################################
-# NexCtl - NexStar control library
+# NexStarCtl - NexStar control library
 # 
 # 
-#		     (c)2013-2014 by Rumen G.Bogdanovski
+#		     (c)2013-2016 by Rumen G.Bogdanovski
 ########################################################
 
 =head1 NAME
@@ -46,8 +46,9 @@
 
 =head1 DESCRIPTION
 
-This module implements the serial commands supported by the Celestron NexStar hand control. This applies to the NexStar GPS,
-NexStar GPS-SA, NexStar iSeries, NexStar SE Series, NexStar GT, CPC, SLT, Advanced-VX, Advanced-GT, and CGE mounts.
+This module implements the serial commands supported by the Celestron NexStar hand control (HC) and SkyWatcher/Orion SynScan HC.
+This applies to the Celestron: NexStar GPS, NexStar GPS-SA, NexStar iSeries, NexStar SE Series, NexStar GT, CPC, SLT, Advanced-VX,
+Advanced-GT, CGE, CGEM etc. SkyWatcher: EQ5, HEQ5, EQ6 (Pro), AZ-EQ5 GT, AZ-EQ6 GT, EQ8 etc. and the corresponding Orion mounts.
 
 Communication to the hand control is 9600 bits/sec, no parity and one stop bit via the RS-232 port on the base of the
 hand control.
@@ -55,7 +56,10 @@ hand control.
 Communication can be established over TCP/IP if nexbridge is running on the computer connected to the telescope.
 
 For extended example how to use this perl module look in to the distribution folder for  nexstarctl/nexstarctl.pl.
-This program is a complete console tool to control NexStar telesctopes based on NexStarCtl module.
+This program is a complete console tool to control NexStar telescopes based on NexStarCtl module.
+
+NOTE: For SkyWatcher/Orion mounts it is highly recommended to enforce protocol version checking (see enforce_protocol_version()) as
+the AUX commands are not supported or may behave erratically.
 
 =cut
 
@@ -73,14 +77,57 @@ if ($^O eq "MSWin32") {
 	eval "use Device::SerialPort"; die $@ if $@;
 }
 
-my $is_tcp=0;
+use constant {
+	VER_1_2    => 0x10200,
+	VER_1_6    => 0x10600,
+	VER_2_2    => 0x20200,
+	VER_2_3    => 0x20300,
+	VER_3_1    => 0x30100,
+	VER_4_10   => 0x40A00,
+	VER_4_37_8 => 0x042508,
+	# All protocol versions
+	VER_AUX  => 0xFFFFFF,
+	# Auto detect version by HC
+	VER_AUTO => 0x0,
+
+	# Protocol vendors
+	VNDR_CELESTRON     => 0x1,
+	VNDR_SKYWATCHER    => 0x2,
+	VNDR_ALL_SUPPORTED => 0x3,   # VNDR_CELESTRON | VNDR_SKYWATCHER
+
+	# version manipulation
+	RELEASE_MASK  => 0xFF0000,
+	REVISION_MASK => 0x00FF00,
+	PATCH_MASK    => 0x0000FF
+};
+
+
+my $is_tcp = 0;
+my $proto_version = VER_AUX;
+my $mount_vendor = VNDR_ALL_SUPPORTED;
+
+# There is no way to tell SkyWatcher from Celestron. Unfortunately both share the
+# same IDs and some Celestron mounts have RTC wile SW does not. That is why the user
+# should decide. Set use_rtc to 1 to enable using the RTC on the mounts that have RTC.
+our $use_rtc = 0;
+
+#  0 => no error
+# -5 => unsupported command
+our $error = 0;
 
 use constant TIMEOUT => 4;
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw( 
 	VERSION
-	
+
+	use_rtc
+	error
+	VER_1_2 VER_1_6 VER_2_2 VER_2_3
+	VER_3_1 VER_4_10 VER_4_37_8 VER_AUX VER_AUTO
+
+	VNDR_CELESTRON VNDR_SKYWATCHER
+
 	DEG2RAD RAD2DEG 
 	notnum precess round 
 	d2hms d2dms d2dm dms2d hms2d
@@ -91,9 +138,12 @@ our @EXPORT = qw(
 	open_telescope_port 
 	close_telescope_port
 	read_telescope 
+	enforce_protocol_version
+	guess_mount_vendor
+	enforce_vendor_protocol
 
 	tc_pass_through_cmd
-	tc_check_align
+	tc_check_align tc_get_orientation
 	tc_goto_rade tc_goto_rade_p
 	tc_goto_azalt tc_goto_azalt_p
 	tc_get_rade tc_get_rade_p
@@ -115,6 +165,8 @@ our @EXPORT = qw(
 	TC_TRACK_ALT_AZ
 	TC_TRACK_EQ_NORTH
 	TC_TRACK_EQ_SOUTH
+	TC_TRACK_EQ
+	TC_TRACK_EQ_PEC
 
 	TC_DIR_POSITIVE
 	TC_DIR_NEGATIVE
@@ -123,13 +175,25 @@ our @EXPORT = qw(
 	TC_AXIS_DE_ALT	
 );
 
-our $VERSION = "0.14";
+our $VERSION = "0.15";
 
 use constant {
+	NX_TC_TRACK_OFF => 0,
+	NX_TC_TRACK_ALT_AZ => 1,
+	NX_TC_TRACK_EQ_NORTH => 2,
+	NX_TC_TRACK_EQ_SOUTH => 3,
+
+	SW_TC_TRACK_OFF => 0,
+	SW_TC_TRACK_ALT_AZ => 1,
+	SW_TC_TRACK_EQ => 2,
+	SW_TC_TRACK_EQ_PEC => 3,
+
 	TC_TRACK_OFF => 0,
 	TC_TRACK_ALT_AZ => 1,
 	TC_TRACK_EQ_NORTH => 2,
 	TC_TRACK_EQ_SOUTH => 3,
+	TC_TRACK_EQ => 4,
+	TC_TRACK_EQ_PEC => 5,
 	
 	TC_DIR_POSITIVE => 1,
 	TC_DIR_NEGATIVE => 0,
@@ -148,20 +212,105 @@ use constant {
 };
 
 my %mounts = (
-	1 => "NexStar GPS Series",
-	3 => "NexStar i-Series",
-	4 => "NexStar i-Series SE",
-	5 => "CGE",
-	6 => "Advanced GT",
-	7 => "SLT",
-	9 => "CPC",
-	10 => "GT",
-	11 => "NexStar 4/5 SE",
-	12 => "NexStar 6/8 SE",
-	14 => "CGEM",
-	20 => "Advanced VX"
+	&VNDR_CELESTRON => {
+		1 => "NexStar GPS Series",
+		3 => "NexStar i-Series",
+		4 => "NexStar i-Series SE",
+		5 => "CGE",
+		6 => "Advanced GT",
+		7 => "SLT",
+		9 => "CPC",
+		10 => "GT",
+		11 => "NexStar 4/5 SE",
+		12 => "NexStar 6/8 SE",
+		14 => "CGEM",
+		20 => "Advanced VX",
+		22 => "Nexstar Evolution"},
+	&VNDR_SKYWATCHER => {
+		0 => "EQ6 Series",
+		1 => "HEQ5 Series",
+		2 => "EQ5 Series",
+		3 => "EQ3 Series",
+		4 => "EQ8 Series",
+		5 => "AZ-EQ6 Series",
+		6 => "AZ-EQ5 Series",
+		160 => "AllView Series",
+		( map { $_ => "AZ Series" } (128..143)),
+		( map { $_ => "DOB Series" } (144..159))
+
+	}
 );
 
+sub release_before($) {
+	my ($release) = @_;
+
+	if ((($proto_version & RELEASE_MASK) >> 16) < $release) {
+		$error = -5;
+	} else {
+		$error = 0;
+	}
+
+	return $error;
+}
+
+sub revision_before($) {
+	my ($revision) = @_;
+
+	if ((($proto_version & REVISION_MASK) >> 8) < $revision) {
+		$error = -5;
+	} else {
+		$error = 0;
+	}
+
+	return $error;
+}
+
+sub patch_before($) {
+	my ($patch) = @_;
+
+	if (($proto_version & PATCH_MASK) < $patch) {
+		$error = -5;
+	} else {
+		$error = 0;
+	}
+
+	return $error;
+}
+
+sub version_before($) {
+	my ($version) = @_;
+
+	if ($proto_version < $version) {
+		$error = -5;
+	} else {
+		$error = 0;
+	}
+
+	return $error;
+}
+
+sub vendor_not($) {
+	my ($vendor) = @_;
+
+	if ($mount_vendor == $vendor) {
+		$error = 0;
+	} else {
+		$error = -5;
+	}
+
+	return $error;
+}
+
+sub vendor_is($) {
+	my ($vendor) = @_;
+
+	if ($mount_vendor == $vendor) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
 =head1 TELESCOPE COMMUNICATION
 
 =over 8
@@ -169,8 +318,8 @@ my %mounts = (
 =item open_telescope_port(port_name)
 
 Opens a communication port to the telescope by name (like "/dev/ttyUSB0") and
-returns it to be used in other finctions. If the port_name has "tcp://" prefix
-the rest of the string is interptered as an IP address and port where to connnect to
+returns it to be used in other functions. If the port_name has "tcp://" prefix
+the rest of the string is interpreted as an IP address and port where to connect to
 (like "tcp://localhost:9999"). In case of error undef is returned.
 
 NOTE: To be used with TCP you need to run nexbridge on the remote computer.
@@ -201,7 +350,7 @@ sub open_telescope_port($) {
 	} else {
 		$port = new Device::SerialPort($portname);
 	}
-	if (! defined $port) {return undef;} 	
+	if (! defined $port) {return undef;}
 	#$port->debug(1);
 	$port->baudrate(9600); 
 	$port->parity("none"); 
@@ -211,6 +360,13 @@ sub open_telescope_port($) {
 	$port->write_settings;
 	$port->read_char_time(0);     # don't wait for each character
 	$port->read_const_time(TIMEOUT*1000);
+
+	$mount_vendor = guess_mount_vendor($port);
+	if (! defined $mount_vendor) {
+		$port->close();
+		return undef;
+	}
+
 	return $port;
 }
 
@@ -230,7 +386,7 @@ sub read_byte($) {
 			$count=$port->read($char,1);
 			alarm 0;
 		};
-		if ($@ and $@ !~ /TimedOut/) {
+		if ($@ and $@ !~ /TimeOut/) {
 			return undef;
 		}
 	}
@@ -239,13 +395,14 @@ sub read_byte($) {
 
 
 =item read_telescope(port, len)
+=item read_telescope(port, len , vl)
 
-Reads data from the telescope. On error or in case less than len bytes are
-read undef is returned.
+Reads data from the telescope. On error or in case less than len bytes are read undef is returned.
+If "vl" is true the function will return success if "#" is read before the len bytes are read.
 
 =cut
-sub read_telescope($$) {
-	my ($port,$len) = @_;
+sub read_telescope {
+	my ($port,$len,$vl) = @_;
 	my $response;
 	my $char;
 	my $count;
@@ -255,8 +412,11 @@ sub read_telescope($$) {
 		if ($count == 0) { return undef; }
 		$total += $count;
 		$response .= $char;
+		if (($vl) and ($char eq "#")) {
+			return $response;
+		}
 	} while ($total < $len);
-	
+
 	# if the last byte is not '#', this means that the device did
 	# not respond and the next byte should be '#' (hopefully)
 	if ($char ne "#") {
@@ -273,7 +433,7 @@ Writes data to the telescope and the result of write is returned.
 =cut
 sub write_telescope($$) {
 	my ($port, $data) = @_;
-	
+
 	return $port->write($data);
 }
 
@@ -285,11 +445,83 @@ Closes the communication port to the telescope.
 
 sub close_telescope_port($) {
 	my ($port) = @_;
-	
+
 	$port->close;
   	undef $port;   
 }
 
+=item enforce_protocol_version(port, ver)
+
+Enforce protocol minimal version checking. If a specific command is not supported by the firmware version given
+in ver, the corresponding tc_*() call will fail as unsupported and return undef (in this case $NexStarCtl::error
+will be set to -5). If version is VER_AUX, version enforcement
+is disabled and all commands are enabled but some may fail, because they may not be supported by the current hand controller
+firmware. To avoid this use VER_AUTO (or omit it) to set the version to the value reported by the currently connected hand
+controller. By default protocol version enforcement is disabled and the unsupported commands will either timeout
+or return erratic results. Because of this, calling enforce_protocol_version() with VER_AUTO or no ver parameter right after
+open_telescope_port() is highly recommended. The predefined versions are: VER_1_2, VER_1_6, VER_2_2, VER_2_3, VER_3_1, VER_4_10 and VER_4_37_8.
+
+NOTE: The non-documented (AUX) commands are available only when the enforcement is disabled.
+
+=cut
+
+sub enforce_protocol_version {
+	my ($port, $ver) = @_;
+
+	if ((defined $ver) and ($ver != 0)) {
+		$proto_version = $ver;
+		return 1;
+	}
+	my ($major,$minor,$subminor) = tc_get_version($port);
+	if (!defined $major) { return undef; }
+	$proto_version = ($major << 16) + ($minor << 8) + $subminor;
+	return 1;
+}
+
+=item guess_mount_vendor(port)
+
+This function guesses the manufacturer of the mount by a slight difference in the protocol. The firmware version command returns 2 bytes for Celestron mounts and 6 bytes for SkyWatcher mounts (since version 4.37.8). On success the guessed value is returned (VNDR_CELESTRON or VNDR_SKYWATCHER). On error undef is returned.
+
+NOTE: SkyWather mounts with firmware before 4.37.8 will be threated as Celestron in this case enforce_vendor_protocol() can be used.
+=cut
+
+sub guess_mount_vendor {
+	my ($port) = @_;
+	return undef if version_before(VER_1_2);
+
+	$port->write("V");
+	my $response = read_telescope($port, 7, 1);
+
+	return undef unless (defined $response);
+
+	if (length($response) == 3) {
+		return VNDR_CELESTRON;
+	} elsif (length($response) == 7) {
+		return VNDR_SKYWATCHER;
+	} else {
+		return undef;
+	}
+}
+
+=item enforce_vendor_protocol(vendor)
+
+This function enforces protocol of the specified vendor to be used overriding the guessed one.
+Valid vendor IDs are VNDR_CELESTRON and VNDR_SKYWATCHER. This way the commands not supported
+by the vendor version of the protocol will fail and $NexStarCtl::error will be set to -5.
+On success the vendor ID is returned otherwise it returns undef.
+
+=cut
+
+sub enforce_vendor_protocol($) {
+	my ($vendor) = @_;
+
+	if (!(VNDR_ALL_SUPPORTED & $vendor)) {
+		return undef;
+	}
+
+	$mount_vendor = $vendor;
+}
+
 #
 #  Telescope Commands
 #
@@ -309,6 +541,8 @@ undef is returned.
 sub tc_check_align($) {
 	my ($port) = @_;
 
+	return undef if version_before(VER_1_2);
+
 	$port->write("J");
 	my $response = read_telescope($port,2);
 	if (defined $response) {
@@ -318,6 +552,28 @@ sub tc_check_align($) {
 	}
 }
 
+=item tc_get_orientation(port)
+
+Get the telescope orientation. "E" or "W" is returned for East and West respectively. If no response received,
+undef is returned.
+
+=cut
+sub tc_get_orientation($) {
+	my ($port) = @_;
+
+	return undef if vendor_not(VNDR_SKYWATCHER);
+	return undef if release_before(3);
+	return undef if revision_before(37);
+
+	$port->write("p");
+	my $response = read_telescope($port,2);
+	if (defined $response) {
+		return substr($response, 0, 1);
+	} else {
+		return undef;
+	}
+}
+
 =item tc_goto_rade(port, ra, de)
 
 =item tc_goto_rade_p(port, ra, de)
@@ -338,9 +594,11 @@ sub tc_goto_rade {
 	}
 	my $nex;
 	if((defined $precise) and ($precise =! 0)) {
+		return undef if version_before(VER_1_6);
 		$nex=dd2pnex($ra, $de);
 		$port->write("r".$nex);
 	} else {
+		return undef if version_before(VER_1_2);
 		$nex=dd2nex($ra, $de);
 		$port->write("R".$nex);
 	}
@@ -375,9 +633,11 @@ sub tc_goto_azalt {
 	}
 	my $nex;
 	if((defined $precise) and ($precise =! 0)) {
+		return undef if version_before(VER_2_2);
 		$nex=dd2pnex($az, $alt);
 		$port->write("b".$nex);
 	} else {
+		return undef if version_before(VER_1_2);
 		$nex=dd2nex($az, $alt);
 		$port->write("B".$nex);
 	}
@@ -408,6 +668,7 @@ sub tc_get_rade {
 	my $de;
 	
 	if((defined $precise) and ($precise =! 0)) {
+		return undef if version_before(VER_1_6);
 		$port->write("e");
 		my $response = read_telescope($port, 18);
 		if (! defined $response) {
@@ -415,6 +676,7 @@ sub tc_get_rade {
 		} 
 		($ra,$de) = pnex2dd($response);
 	} else {
+		return undef if version_before(VER_1_2);
 		$port->write("E");
 		my $response = read_telescope($port, 10);
 		if (! defined $response) {
@@ -445,6 +707,7 @@ sub tc_get_azalt {
 	my $alt;
 	
 	if((defined $precise) and ($precise =! 0)) {
+		return undef if version_before(VER_2_2);
 		$port->write("z");
 		my $response = read_telescope($port, 18);
 		if (! defined $response) {
@@ -452,6 +715,7 @@ sub tc_get_azalt {
 		} 
 		($az,$alt) = pnex2dd($response);
 	} else {
+		return undef if version_before(VER_1_2);
 		$port->write("Z");
 		my $response = read_telescope($port, 10);
 		if (! defined $response) {
@@ -471,13 +735,21 @@ sub tc_get_azalt_p {
 
 =item tc_sync_rade_p(port, ra, de)
 
-Syncs the telescope to RA/DEC coordinates ra, de (in decimal degrees). Function tc_goto_azalt_p uses precise sync.
+Syncs the telescope to RA/DEC coordinates ra, de (in decimal degrees). Function tc_sync_rade_p uses precise sync.
 If RA is not in [0;360] function returns -1. If DEC is not in [-90;90] -2 is returned.
 If no response received, undef is returned.
 
 =cut
 sub tc_sync_rade {
 	my ($port, $ra, $de, $precise) = @_;
+
+	if(vendor_is(VNDR_SKYWATCHER)) {
+		return undef if release_before(3);
+		return undef if revision_before(37);
+	} else {
+		return undef if version_before(VER_4_10);
+	}
+
 	if (($ra < 0) or ($ra > 360)) {
 		return -1;
 	}
@@ -512,6 +784,8 @@ Returns 1 if GOTO is in progress else 0 is returned. If no response received, un
 sub tc_goto_in_progress($) {
 	my ($port) = @_;
 
+	return undef if version_before(VER_1_2);
+
 	$port->write("L");
 	my $response = read_telescope($port, 2);
 	if (defined $response) {
@@ -528,6 +802,8 @@ Cancels the GOTO operation. On success 1 is returned. If no response received, u
 =cut
 sub tc_goto_cancel($) {
 	my ($port) = @_;
+
+	return undef if version_before(VER_1_2);
 		        
 	$port->write("M");
 	my $response = read_telescope($port, 1);
@@ -540,13 +816,15 @@ sub tc_goto_cancel($) {
 
 =item tc_echo(port, char)
 
-Checks the communication with the telecope. This function sends char to the telescope and 
+Checks the communication with the telescope. This function sends char to the telescope and 
 returns the echo received. If no response received, undef is returned.
 
 =cut
 sub tc_echo($$) {
 	my ($port, $char) = @_;
 
+	return undef if version_before(VER_1_2);
+
 	$port->write("K".substr($char, 0, 1));
 	my $response = read_telescope($port, 2);
 	if (defined $response) {
@@ -565,6 +843,8 @@ If no response received, undef is returned.
 sub tc_get_model($) {
 	my ($port) = @_;
 
+	return undef if version_before(VER_2_2);
+
 	$port->write("m");
 	my $response = read_telescope($port, 2);
 	if (defined $response) {
@@ -576,21 +856,28 @@ sub tc_get_model($) {
 
 =item tc_get_version(port)
 
-This function returns the mount version as a number. See CELESTRON documentation.
+This function returns the mount version as a strng or as major and minor numbers in array context  See CELESTRON documentation.
 If no response received, undef is returned.
 
 =cut
 sub tc_get_version($) {
 	my ($port) = @_;
 
+	return undef if version_before(VER_1_2);
+
 	$port->write("V");
-	my $response = read_telescope($port, 3);
+	my $response = read_telescope($port, 7, 1);
 	if (defined $response) {
-		return ord(substr($response, 0, 1)).
-			   ".".ord(substr($response, 1, 1));
-	} else {
-		return undef;
+		if (length($response) == 3) {
+			return wantarray ? (ord(substr($response, 0, 1)),ord(substr($response, 1, 1)))
+				         : ord(substr($response, 0, 1)).".".ord(substr($response, 1, 1));
+		} elsif (length($response) == 7) {
+			return wantarray ? (hex(substr($response, 0, 2)),hex(substr($response, 2, 2)),hex(substr($response, 4, 2)))
+				         : hex(substr($response, 0, 2)).".".hex(substr($response, 2, 2)).".".hex(substr($response, 4, 2));
+		}
 	}
+
+	return undef;
 }
 
 
@@ -609,6 +896,8 @@ If no response received, undef is returned.
 sub tc_get_location {
 	my ($port,$str) = @_;
 
+	return undef if version_before(VER_2_3);
+
 	$port->write("w");
 	my $response = read_telescope($port, 9);
 	if (! defined $response) {
@@ -664,6 +953,8 @@ sub tc_set_location {
 	my ($port,$lon,$lat) = @_;
 	my $issouth = 0;
 	my $iswest = 0;
+
+	return undef if version_before(VER_2_3);
 	
 	if ($lon < 0) {
 		$lon *= -1;
@@ -701,7 +992,7 @@ sub tc_set_location {
 
 =item tc_get_time(port)
 
-This function returns the stored time (in unixtime format), timezone (in hours) and daylight saving time(0|1).
+This function returns the stored time (in Unix time format), timezone (in hours) and daylight saving time(0|1).
 If no response received, undef is returned.
 
 =item tc_get_time_str(port)
@@ -714,6 +1005,8 @@ If no response received, undef is returned.
 sub tc_get_time {
 	my ($port,$str) = @_;
 
+	return undef if version_before(VER_2_3);
+
 	$port->write("h");
 	my $response = read_telescope($port, 9);
 	if (! defined $response) {
@@ -746,15 +1039,20 @@ sub tc_get_time_str {
 
 =item tc_set_time(port, time, timezone, daylightsaving)
 
-This function sets the time (in unixtime format), timezone (in hours) and daylight saving time(0|1).
+This function sets the time (in Unix time format), timezone (in hours) and daylight saving time(0|1).
 On success 1 is returned.
 If no response received, undef is returned. If the mount is known to have RTC
-(currently only CGE and AdvancedVX) the date/time is set to RTC too.
+(currently only CGE and AdvancedVX) and NexStarCtl::use_rtc is defined and != 0 (default is 0), the date/time is
+set to the RTC too.
+
+NOTE: Do not set NexStarCtl::use_rtc if the mount is SkyWatcher otherwise tc_set_time() may fail.
 
 =cut
 sub tc_set_time {
 	my ($port, $time, $tz, $dst) = @_;
 
+	return undef if version_before(VER_2_3);
+
 	my $timezone = $tz;
 	$tz += 256 if ($tz < 0);
 
@@ -784,6 +1082,11 @@ sub tc_set_time {
 		return undef;
 	}
 
+	# return success if the RTC is not used
+	if (($use_rtc == 0) or ($use_rtc == undef)) {
+		return 1;
+	}
+
 	my $model = tc_get_model($port);
 	# If the mount has RTC set date/time to RTC too
 	# I only know CGE(5) and AdvancedVX(20) to have RTC
@@ -817,38 +1120,130 @@ sub tc_set_time {
 =item tc_get_tracking_mode(port)
 
 Reads the tracking mode of the mount and returns one of the folowing:
-TC_TRACK_OFF, TC_TRACK_ALT_AZ, TC_TRACK_EQ_NORTH, TC_REACK_EQ_SOUTH.
+TC_TRACK_OFF, TC_TRACK_ALT_AZ, TC_TRACK_EQ_NORTH (Celestron only), TC_TRACK_EQ_SOUTH (Celestron only),
+TC_TRACK_EQ (SkyWatcher only) and TC_TRACK_EQ_PEC (Sky-Watcher only).
 If no response received, undef is returned.
 
 =cut
 sub tc_get_tracking_mode($) {
 	my ($port) = @_;
+	my $mode;
+
+	return undef if version_before(VER_2_3);
 
 	$port->write("t");
 	my $response = read_telescope($port, 2);
 	if (defined $response) {
-		return ord(substr($response, 0, 1));
+		$mode = ord(substr($response, 0, 1));
 	} else {
 		return undef;
 	}
+
+	# Sky-Watcher mount
+	if(vendor_is(VNDR_SKYWATCHER)) {
+		if ($mode == SW_TC_TRACK_OFF) {
+			return TC_TRACK_OFF;
+
+		} elsif ($mode == SW_TC_TRACK_ALT_AZ) {
+			return TC_TRACK_ALT_AZ;
+
+		} elsif ($mode == SW_TC_TRACK_EQ) {
+			return TC_TRACK_EQ;
+
+		} elsif ($mode == SW_TC_TRACK_EQ_PEC) {
+			return TC_TRACK_EQ_PEC;
+		}
+	# Celestron mount
+	} else {
+		if ($mode == NX_TC_TRACK_OFF) {
+			return TC_TRACK_OFF;
+
+		} elsif ($mode == NX_TC_TRACK_ALT_AZ) {
+			return TC_TRACK_ALT_AZ;
+
+		} elsif ($mode == NX_TC_TRACK_EQ_NORTH) {
+			return TC_TRACK_EQ_NORTH;
+
+		} elsif ($mode == NX_TC_TRACK_EQ_SOUTH) {
+			return TC_TRACK_EQ_SOUTH;
+		}
+	}
+	return undef;
 }
 
 =item tc_set_tracking_mode(port, mode)
 
 Sets the tracking mode of the mount to one of the folowing:
-TC_TRACK_OFF, TC_TRACK_ALT_AZ, TC_TRACK_EQ_NORTH, TC_REACK_EQ_SOUTH.
-If the mode is not one of the listed -1 is returned.
+TC_TRACK_OFF, TC_TRACK_ALT_AZ, TC_TRACK_EQ_NORTH, TC_TRACK_EQ_SOUTH, TC_TRACK_EQ and TC_TRACK_EQ_PEC.
+For Sky-Watcher TC_TRACK_EQ_NORTH, TC_TRACK_EQ_SOUTH and TC_TRACK_EQ do the same thing. For Celestron
+TC_TRACK_EQ is interpreted as TC_TRACK_EQ_NORTH or TC_TRACK_EQ_SOUTH depending on the geographic latitude
+of the location (Northern or Southern hemisphere). If the mode is not one of the listed -1 is returned.
 If no response received, undef is returned.
 
 =cut
 sub tc_set_tracking_mode($$) {
 	my ($port,$mode) = @_;
+	my $_mode;
 
-	if (($mode < 0) or ($mode > 3)) {
-		return -1;
+	return undef if version_before(VER_1_6);
+
+	# Sky-Watcher mount
+	if(vendor_is(VNDR_SKYWATCHER)) {
+		if ($mode == TC_TRACK_OFF) {
+			$_mode = TC_TRACK_OFF;
+
+		} elsif ($mode == TC_TRACK_ALT_AZ) {
+			$_mode = SW_TC_TRACK_ALT_AZ;
+
+		} elsif (($mode == TC_TRACK_EQ) or
+				($mode == TC_TRACK_EQ_NORTH) or
+				($mode == TC_TRACK_EQ_SOUTH)) {
+			$_mode = SW_TC_TRACK_EQ;
+
+		} elsif ($mode == TC_TRACK_EQ_PEC) {
+			$_mode = SW_TC_TRACK_EQ_PEC;
+
+		} else {
+			return -1;
+		}
+
+	# Celestron mount
+	} else {
+		if ($mode == TC_TRACK_OFF) {
+			$_mode = NX_TC_TRACK_OFF;
+
+		} elsif ($mode == TC_TRACK_ALT_AZ) {
+			$_mode = NX_TC_TRACK_ALT_AZ;
+
+		} elsif ($mode == TC_TRACK_EQ_NORTH) {
+			$_mode = NX_TC_TRACK_EQ_NORTH;
+
+		} elsif ($mode == TC_TRACK_EQ_SOUTH) {
+			$_mode = NX_TC_TRACK_EQ_SOUTH;
+
+		} elsif ($mode == TC_TRACK_EQ) {
+			my (undef, $lat) = tc_get_location($port);
+			if (!defined $lat) {
+				return -1;
+			}
+			if ($lat < 0) {
+				$_mode = NX_TC_TRACK_EQ_SOUTH;
+			} else {
+				$_mode = NX_TC_TRACK_EQ_NORTH;
+			}
+
+		} elsif ($mode == TC_TRACK_EQ_PEC) {
+			# unsupported on Celestron
+			$error = -5;
+			return undef;
+
+		} else {
+			return -1;
+		}
 	}
+
 	$port->write("T");
-	$port->write(chr($mode));
+	$port->write(chr($_mode));
 	my $response = read_telescope($port, 1);
 	if (defined $response) {
 		return 1;
@@ -871,6 +1266,13 @@ If no response received, undef is returned.
 =cut
 sub tc_slew_fixed {
 	my ($port,$axis,$direction,$rate) = @_;
+
+	if(vendor_is(VNDR_SKYWATCHER)) {
+		return undef if release_before(3);
+		return undef if revision_before(1);
+	} else {
+		return undef if version_before(VER_1_6);
+	}
 	
 	if ($axis>0) { 
 		$axis = _TC_AXIS_RA_AZM;
@@ -911,6 +1313,13 @@ On success 1 is returned. If no response received, undef is returned.
 sub tc_slew_variable {
 	my ($port,$axis,$direction,$rate) = @_;
 
+	if(vendor_is(VNDR_SKYWATCHER)) {
+		return undef if release_before(3);
+		return undef if revision_before(1);
+	} else {
+		return undef if version_before(VER_1_6);
+	}
+
 	if ($axis>0) { 
 		$axis = _TC_AXIS_RA_AZM;
 	} else {
@@ -945,7 +1354,7 @@ If the mount is not known undef is returned.
 
 sub get_model_name($) {
 	my ($model_id) = @_;
-	return $mounts{$model_id};
+	return $mounts{$mount_vendor}{$model_id};
 }
 
 =back
@@ -969,6 +1378,8 @@ If no response received, undef is returned.
 sub tc_get_autoguide_rate($$) {
 	my ($port,$axis) = @_;
 
+	return undef if version_before(VER_AUX);
+
 	if ($axis > 0) {
 		$axis = _TC_AXIS_RA_AZM;
 	} else {
@@ -998,6 +1409,8 @@ If no response received, undef is returned.
 sub tc_set_autoguide_rate($$$) {
 	my ($port,$axis,$rate) = @_;
 
+	return undef if version_before(VER_AUX);
+
 	if ($axis > 0) {
 		$axis = _TC_AXIS_RA_AZM;
 	} else {
@@ -1043,6 +1456,9 @@ If no response received, undef is returned.
 sub tc_get_backlash($$$) {
 	my ($port,$axis,$direction) = @_;
 
+	return undef if vendor_not(VNDR_CELESTRON);
+	return undef if version_before(VER_AUX);
+
 	if ($axis > 0) {
 		$axis = _TC_AXIS_RA_AZM;
 	} else {
@@ -1076,6 +1492,9 @@ If no response received, undef is returned.
 sub tc_set_backlash($$$$) {
 	my ($port,$axis,$direction,$backlash) = @_;
 
+	return undef if vendor_not(VNDR_CELESTRON);
+	return undef if version_before(VER_AUX);
+
 	if ($axis > 0) {
 		$axis = _TC_AXIS_RA_AZM;
 	} else {
@@ -1144,7 +1563,7 @@ sub notnum($)
 
 =item precess(ra0, dec0, equinox0, equinox1)
 
-Precesses coordinates ra0 and dec0 from equinox0 to equinox1 and returns the caclculated ra1 and dec1. 
+Precesses coordinates ra0 and dec0 from equinox0 to equinox1 and returns the calculated ra1 and dec1. 
 Where ra and dec should be in decimal degrees and equinox should be in years (and fraction of the year).
 
 =cut
