@@ -19,7 +19,7 @@ use if $^O eq "MSWin32", "Win32::Console::ANSI";
 use Term::ANSIColor qw(:constants);
 $Term::ANSIColor::AUTORESET = 1;
 
-my $VERSION = "0.2";
+my $VERSION = "0.5";
 
 my $port;
 my $verbose;
@@ -37,24 +37,29 @@ sub print_help() {
 	      "       $N gettime [telescope]\n".
 	      "       $N setlocation LON LAT [telescope]\n".
 	      "       $N getlocation [telescope]\n".
-	      "       $N settrack [north|south|azalt|off] [telescope]\n".
+	      "       $N settrack [equatorial|north|south|azalt|pec|off] [telescope]\n".
 	      "       $N gettrack [telescope]\n".
 	      "       $N goto RA DE [telescope]\n".
 	      "       $N gotoaz AZ ALT [telescope]\n".
+	      "       $N sync RA DE [telescope]\n".
 	      "       $N getrade [telescope]\n".
 	      "       $N getazalt [teelescope]\n".
 	      "       $N abort [telescope]\n".
 	      "       $N status [telescope]\n".
+	      "       $N netscan\n".
 	      "options:\n".
 	      "       -v verbose output\n".
 	      "       -h print this help\n".
 	      "[telescope]:\n".
 	      "       The telescope port could be specified with this parameter or TELESCOPE_PORT\n".
-	      "       environment can be set. Defaults depend on the operating system:\n".
+	      "       environment variable can be set. Defaults depend on the operating system:\n".
 	      "          Linux: /dev/ttyUSB0\n".
 	      "          MacOSX: /dev/cu.usbserial\n".
 	      "          Solaris: /dev/ttya\n".
-	      "          Windows: COM1\n";
+	      "          Windows: COM1\n".
+	      "       The network connected telescopes can be listed with \"$N netscan\"\n".
+	      "       and the value of TELESCOPE_PORT for each telescope is given.\n\n".
+	      "NOTE: RA and DE should be secified in J2000 and are returned in J2000.\n\n";
 }
 
 
@@ -80,6 +85,13 @@ sub init_telescope {
 		return undef;
 	}
 
+	my $res = enforce_protocol_version($dev);
+	if ($res == undef) {
+		print RED "Communication with the telescope on port $tport failed: $!\n";
+		close_telescope_port($dev);
+		return undef;
+	}
+
 	$verbose && print GREEN "The telescope port $tport is open.\n";
 
 	if (! $check) {
@@ -166,19 +178,26 @@ sub status {
 
 	my $tracking = tc_get_tracking_mode($dev);
 	if (!defined $tracking) {
-		print RED "status: Error geting tracking mode. $!\n";
-		close_telescope_port($dev);
-		return undef;
-	}
-
-	if (($status == 0) && ($tracking == TC_TRACK_OFF)) {
-		print "Telescope is not tracking.\n";
-	} elsif (($status == 0) && ($tracking != TC_TRACK_OFF)) {
-		print "Telescope is tracking.\n";
+		if ($NexStarCtl::error != -5) {
+			print RED "status: Error geting tracking mode. $!\n";
+			close_telescope_port($dev);
+			return undef;
+		} else {  # get tracking mode is unsupported on this mount
+			if ($status == 0) {
+				print "Telescope is either tracking or not, but GOTO is not in progress.\n";
+			} else {
+				print "GOTO is in progress.\n";
+			}
+		}
 	} else {
-		print "GOTO is in progress.\n";
+		if (($status == 0) && ($tracking == TC_TRACK_OFF)) {
+			print "Telescope is not tracking.\n";
+		} elsif (($status == 0) && ($tracking != TC_TRACK_OFF)) {
+			print "Telescope is tracking.\n";
+		} else {
+			print "GOTO is in progress.\n";
+		}
 	}
-
 	close_telescope_port($dev);
 	return 1;
 }
@@ -234,7 +253,7 @@ sub settime {
 			$port = $params[0];
 		}
 		$time=time();
-	    $isdst = (localtime($time))[-1];
+		$isdst = (localtime($time))[-1];
 		$tz = int((timegm(localtime($time)) - $time) / 3600);
 		$tz = $tz-1 if ($isdst);
 
@@ -372,13 +391,16 @@ sub settrack {
 	}
 
 	my $tracking;
-
-	if ($mode eq "south") {
+	if ($mode eq "equatorial") {
+		$tracking = TC_TRACK_EQ;
+	} elsif ($mode eq "south") {
 		$tracking = TC_TRACK_EQ_SOUTH;
 	} elsif ($mode eq "north") {
 		$tracking = TC_TRACK_EQ_NORTH;
 	} elsif ($mode eq "azalt") {
 		$tracking = TC_TRACK_ALT_AZ;
+	} elsif ($mode eq "pec") {
+		$tracking = TC_TRACK_EQ_PEC;
 	} elsif ($mode eq "off") {
 		$tracking = TC_TRACK_OFF;
 	} else {
@@ -433,6 +455,10 @@ sub gettrack {
 		print "Tracking: Equatorial North\n";
 	} elsif ($tracking == TC_TRACK_ALT_AZ) {
 		print "Tracking: Aaltazimuthal\n";
+	} elsif ($tracking == TC_TRACK_EQ) {
+		print "Tracking: Equatorial\n";
+	} elsif ($tracking == TC_TRACK_EQ_PEC) {
+		print "Tracking: Equatorial + PEC\n";
 	} else {
 		print "Tracking: Unknown\n";
 	}
@@ -503,6 +529,65 @@ sub gotoeq {
 	return 1;
 }
 
+sub sync {
+	my @params = @_;
+	my $ra;
+	my $de;
+
+	if ($#params == 1) {
+		$ra = $params[0];
+		$de = $params[1];
+
+	} elsif ($#params == 2) {
+		$ra = $params[0];
+		$de = $params[1];
+		$port = $params[2];
+
+	} else {
+		print RED "sync: Wrong parameters.\n";
+		return undef;
+	}
+
+	# convert strings $ra/$de to $rad/$ded in decimal degrees
+	my $rad = hms2d($ra);
+	if ((!defined $rad) or ($rad > 360) or ($rad < 0)) {
+		print RED "sync: Wrong rightascension.\n";
+		return undef;
+	}
+
+	my $ded = dms2d($de);
+	if ((!defined $ded) or ($ded > 90) or ($ded < -90)) {
+		print RED "sync: Wrong declination.\n";
+		return undef;
+	}
+
+	# if telescope is slewing do nothing
+	# so set second parameter to 1
+	my $dev = init_telescope($port, 1);
+	return undef if (! defined $dev);
+
+	my $align = tc_check_align($dev);
+	if (!defined $align) {
+		print RED "sync: Error reading from telescope.\n";
+		return undef;
+	} elsif ($align == 0) {
+		print RED "The telescope is NOT aligned. Please align before sync.\n";
+		return undef;
+	}
+	$verbose && print GREEN "The telescope is aligned.\n";
+
+	$verbose && print "sync: ra = $rad, dec = $ded\n";
+
+	if (! tc_sync_rade_p($dev, $rad, $ded)) {
+		print RED "sync: Failed. $!\n";
+		close_telescope_port($dev);
+		return undef;
+	}
+
+	close_telescope_port($dev);
+	return 1;
+}
+
 
 sub getrade {
 	my @params = @_;
@@ -651,6 +736,37 @@ sub abort {
 }
 
 
+sub netscan {
+	eval "use Net::Bonjour";
+	if ($@) {
+		print RED "This feature is disabled to enable it Net::Bonjour should be installed!\n";
+		return undef;
+	}
+	my $res = Net::Bonjour->new('nexbridge');
+
+	$res->discover();
+
+	if ( $res->entries == 0) {
+		print "\nNo telescopes discovered!\n";
+		print YELLOW "To discover the network connected telescopes they should be\n";
+		print YELLOW "exported via Bonjour with nexbridge (See perldoc NexStarCtl)\n\n";
+		return 1;
+	}
+
+	print "\n";
+	printf "%-30s %s\n", "NAME", "TELESCOPE_PORT";
+	printf "%-30s %s\n", "===============", "==============";
+
+	$res->discover();
+	foreach my $entry ($res->entries) {
+		printf "%-30s %s://%s:%s (%s)\n", $entry->name, $res->protocol, $entry->hostname, $entry->port, $entry->address;
+	}
+	print "\n";
+
+        return 1;
+}
+
+
 sub main() {
 	my %options = ();
 
@@ -747,6 +863,14 @@ sub main() {
 		$verbose && print GREEN "GOTO ra/de succeeded.\n";
 		exit 0;
 
+	} elsif ($command eq "sync") {
+		if (! sync(@ARGV)) {
+			$verbose && print RED "SYNC ra/de returned error.\n";
+			exit 1;
+		}
+		$verbose && print GREEN "SYNC ra/de succeeded.\n";
+		exit 0;
+
 	} elsif ($command eq "gotoaz") {
 		if (! gotoaz(@ARGV)) {
 			$verbose && print RED "GOTO az/alt returned error.\n";
@@ -787,6 +911,14 @@ sub main() {
 		$verbose && print GREEN "Status succeeded.\n";
 		exit 0;
 
+	} elsif ($command eq "netscan") {
+		if (! netscan()) {
+			$verbose && print RED "Netscan returned error.\n";
+			exit 1;
+		}
+		$verbose && print GREEN "Netscan succeeded.\n";
+		exit 0;
+
 	} elsif ($command eq "-h") {
 		print_help();
 		exit 0;
